package com.hs.admin.mapper;

import com.hs.admin.model.SysUserRoleRs;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface SysUserRoleRsMapper {
    int deleteByPrimaryKey(Integer id);

    int insert(SysUserRoleRs record);

    SysUserRoleRs selectByPrimaryKey(Integer id);

    List<SysUserRoleRs> selectAll();

    int updateByPrimaryKey(SysUserRoleRs record);

    int deleteByUser(@Param("userId") Long userId);

    SysUserRoleRs selectByUser(@Param("userId") Long userId,@Param("roleId") Long roleId);
}